@@ -10,7 +10,7 @@
 #include "type.h"
 
 /* Int */
-int32_t unpack_int(pTHX_ char *input, STRLEN len, STRLEN *pos)
+int32_t unpack_int(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos)
 {
     if (UNLIKELY(len - *pos < 4))
         croak("unpack_int: input too short. Data corrupted?");
@@ -23,10 +23,10 @@ STRLEN pack_int(pTHX_ SV *dest, int32_t number)
 {
     union {
         int32_t number;
-        char bytes[4];
+        unsigned char bytes[4];
     } int_or_bytes;
     int_or_bytes.number = htonl(number);
-    sv_catpvn(dest, int_or_bytes.bytes, 4);
+    sv_catpvn(dest, (char*)int_or_bytes.bytes, 4);
     return SvCUR(dest)-4;
 }
 
@@ -36,7 +36,7 @@ void set_packed_int(pTHX_ SV *dest, STRLEN pos, int32_t number)
     char *ptr;
     union {
         int32_t number;
-        char bytes[4];
+        unsigned char bytes[4];
     } int_or_bytes;
     int_or_bytes.number = htonl(number);
     ptr = SvPV(dest, len);
@@ -45,7 +45,7 @@ void set_packed_int(pTHX_ SV *dest, STRLEN pos, int32_t number)
 }
 
 /* Short */
-int unpack_short_nocroak(pTHX_ char *input, STRLEN len, STRLEN *pos, uint16_t *out)
+int unpack_short_nocroak(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos, uint16_t *out)
 {
     if (UNLIKELY(len - *pos < 2))
         return -1;
@@ -54,7 +54,7 @@ int unpack_short_nocroak(pTHX_ char *input, STRLEN len, STRLEN *pos, uint16_t *o
     return 0;
 }
 
-uint16_t unpack_short(pTHX_ char *input, STRLEN len, STRLEN *pos)
+uint16_t unpack_short(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos)
 {
     uint16_t out;
     if (UNLIKELY(unpack_short_nocroak(aTHX_ input, len, pos, &out) != 0))
@@ -66,14 +66,14 @@ void pack_short(pTHX_ SV *dest, uint16_t number)
 {
     union {
         uint16_t number;
-        char bytes[2];
+        unsigned char bytes[2];
     } short_or_bytes;
     short_or_bytes.number = htons(number);
-    sv_catpvn(dest, short_or_bytes.bytes, 2);
+    sv_catpvn(dest, (char*)short_or_bytes.bytes, 2);
 }
 
 /* Bytes */
-int unpack_bytes(pTHX_ char *input, STRLEN len, STRLEN *pos, char **output, STRLEN *outlen)
+int unpack_bytes(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos, unsigned char **output, STRLEN *outlen)
 {
     int32_t bytes_length = unpack_int(aTHX_ input, len, pos);
     if (bytes_length < 0) {
@@ -90,40 +90,40 @@ int unpack_bytes(pTHX_ char *input, STRLEN len, STRLEN *pos, char **output, STRL
     return 0;
 }
 
-SV *unpack_bytes_sv(pTHX_ char *input, STRLEN len, STRLEN *pos)
+SV *unpack_bytes_sv(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos)
 {
-    char *bytes;
+    unsigned char *bytes;
     STRLEN bytes_len;
 
     if (unpack_bytes(aTHX_ input, len, pos, &bytes, &bytes_len) == 0) {
-        return newSVpvn(bytes, bytes_len);
+        return newSVpvn((char*)bytes, bytes_len);
     } else {
         return &PL_sv_undef;
     }
 }
 
 /* String */
-int unpack_string_nocroak(pTHX_ char *input, STRLEN len, STRLEN *pos, char **output, STRLEN *outlen)
+int unpack_string_nocroak(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos, char **output, STRLEN *outlen)
 {
     uint16_t string_length = unpack_short(aTHX_ input, len, pos);
 
     if (UNLIKELY(len - *pos < string_length))
         return -1;
 
-    *output = input + *pos;
+    *output = (char*)(input + *pos);
     *outlen = string_length;
     *pos += string_length;
 
     return 0;
 }
 
-void unpack_string(pTHX_ char *input, STRLEN len, STRLEN *pos, char **output, STRLEN *outlen)
+void unpack_string(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos, char **output, STRLEN *outlen)
 {
     if (UNLIKELY(unpack_string_nocroak(aTHX_ input, len, pos, output, outlen)) != 0)
         croak("unpack_string: input invalid");
 }
 
-SV *unpack_string_sv(pTHX_ char *input, STRLEN len, STRLEN *pos)
+SV *unpack_string_sv(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos)
 {
     char *string;
     STRLEN str_len;
@@ -131,7 +131,7 @@ SV *unpack_string_sv(pTHX_ char *input, STRLEN len, STRLEN *pos)
     return newSVpvn_utf8(string, str_len, 1);
 }
 
-SV *unpack_string_sv_hash(pTHX_ char *input, STRLEN len, STRLEN *pos, U32 *hashout)
+SV *unpack_string_sv_hash(pTHX_ unsigned char *input, STRLEN len, STRLEN *pos, U32 *hashout)
 {
     char *string;
     STRLEN str_len;
